from fridaay import Pipe
from .data import FrameData

import toga
from toga.sources import TreeSource
from toga.sources.tree_source import Node
from toga.constants import COLUMN, ROW
from toga.style import Pack

class FrameView():

    def __init__(self, app, name, raw_data):
        self.app = app
        self.name = name
        self.data = FrameData(raw_data)
        self.startup()

    def startup(self):
        # Set up main window

        self.app.window_counter += 1
        self.window = toga.Window(title=self.name)
        # Both self.windows.add() and self.windows += work:
        self.app.windows += self.window

        # Label to show responses.
        self.label = toga.Label('Ready.', style=Pack(padding=10))

        self.table = toga.Table(
            headings=self.data.columns,
            data=self.data,
            multiple_select=True,
            style=Pack(flex=1, padding_left=5),
        )

        # Outermost box
        outer_box = toga.Box(
            children=[self.table, self.label],
            style=Pack(
                flex=1,
                direction=COLUMN,
            )
        )

        # Add the content on the main window
        self.window.content = outer_box

        # Show the main window
        self.window.show()



from fridaay import Pipe
from .io import obj2tree
from .view import FrameView

import toga
from toga.sources import TreeSource
from toga.sources.tree_source import Node
from toga.constants import COLUMN, ROW
from toga.style import Pack

class PipeBookDoc():

    def __init__(self, app, name, yml):
        self.app = app
        self.name = name
        self.yaml = yml
        self.source = obj2tree(yml)
        self.pipe = Pipe(app.registry, yml)
        self.frames = []
        self.startup()

    # Pipe Run functions

    async def do_run(self, widget):
        self.pipe.run()
        self.frames = [FrameView(self.app, n, d) for n,d in self.pipe.data.items()]

    # Table callback functions
    def on_select_handler(self, widget, node):
        if node is not None and node.name:
            self.label.text = f'You selected node: {node.name}'
            self.btn_remove.enabled = True
        else:
            self.label.text = 'No node selected'
            self.btn_remove.enabled = False

    # Button callback functions
    def insert_handler(self, widget, **kwargs):
        self.tree.data

    def remove_handler(self, widget, **kwargs):
        selection = self.tree.selection
        if selection.title:
            self.tree.data.remove(selection)

    def startup(self):
        # Set up doc window

        self.app.window_counter += 1
        self.window = toga.Window(title=self.name)
        # Both self.windows.add() and self.windows += work:
        self.app.windows += self.window

        # Label to show responses.
        self.label = toga.Label('Ready.', style=Pack(padding=10))

        self.tree = toga.Tree(
            headings=self.source._accessors,
            style=Pack(flex=1)
        )
        self.tree.data = self.source

        # Buttons
        btn_style = Pack(flex=1, padding=10)

        self.btn_insert = toga.Button('Insert Row', on_press=self.insert_handler, style=btn_style)
        self.btn_remove = toga.Button('Remove Row', enabled=False, on_press=self.remove_handler, style=btn_style)
        self.btn_box = toga.Box(children=[self.btn_insert, self.btn_remove], style=Pack(direction=ROW))
        self.btn_run = toga.Button('Run and Show Frames', on_press=self.do_run, style=btn_style)

        # Outermost box
        outer_box = toga.Box(
            children=[self.btn_run, self.btn_box, self.tree, self.label],
            style=Pack(
                flex=1,
                direction=COLUMN,
            )
        )

        # Add the content on the main window
        self.window.content = outer_box

        # Show the main window
        self.window.show()



import traceback
from pathlib import Path
from fridaay import *
from .doc import PipeBookDoc
import importlib.resources

import toga
from toga.constants import COLUMN
from toga.style import Pack
APP_NAME = 'PipeBook'
APP_ID = 'com.igwet.app.pipebook'
APP_ICON = '/Users/ernest/Developer/pipebook/pipebook/resources/pipebook2.png'
DEMO_PIPE="demo_pets"

class PipeBookApp(toga.App):

    def do_clear(self, widget, **kwargs):
        self.label.text = "Ready."

    def open_document(self, fname):
        name = str(fname)
        self.label.text = f"File to open: {name}"
        path, ext = name.split(".")
        yml = load_yaml(path)
        doc = PipeBookDoc(self, name, yml)
        return doc

    def load_demo(self, name=DEMO_PIPE):
        pipe_path = path_resource(PKG_ID, PIPE_FOLDER)
        yml = load_yaml(name, pipe_path)
        doc = PipeBookDoc(self, name, yml)
        return doc

    async def action_info_dialog(self, widget):
        await self.main_window.info_dialog(APP_NAME, f'Welcome to {APP_NAME}')
        self.label.text = 'The next-generation data notebook for resilient, production-ready data pipelines.'

    async def action_open_file_filtered_dialog(self, widget):
        try:
            fname = await self.main_window.open_file_dialog(
                title=f"Open {APP_NAME} YAML file",
                multiselect=False,
                file_types=['yml'],
            )
            if fname is not None:
                self.open_document(fname)
            else:
                self.label.text = "No file selected!"
        except ValueError:
            self.label.text = "Open file dialog was canceled"

    async def exit_handler(self, app):
        # Return True if app should close, and False if it should remain open
        if await self.main_window.confirm_dialog(APP_NAME, 'Are you sure you want to quit?'):
            print(f"Label text was '{self.label.text}' when you quit the app")
            return True
        else:
            self.label.text = 'Exit canceled'
            return False

    def set_window_label_text(self, num_windows):
        self.window_label.text = f"{num_windows} secondary window(s) open"

    def startup(self):
        # Set up main window
        self.registry = Registry()
        self.main_window = toga.MainWindow(title=self.name)
        self.on_exit = self.exit_handler

        # Label to show responses.
        self.label = toga.Label('Ready.', style=Pack(padding_top=20))
        self.window_label = toga.Label('', style=Pack(padding_top=20))
        self.window_counter = 0
        self.close_attempts = set()
        self.set_window_label_text(0)

        btn_style = Pack(flex=1)
        btn_info = toga.Button('Info', on_press=self.action_info_dialog, style=btn_style)
        btn_open_filtered = toga.Button(
            'Open PipeBook YAML',
            on_press=self.action_open_file_filtered_dialog,
            style=btn_style
        )
        btn_clear = toga.Button('Clear', on_press=self.do_clear, style=btn_style)

        # Outermost box
        box = toga.Box(
            children=[
                btn_info,
                btn_open_filtered,
                btn_clear,
                self.label,
                self.window_label
            ],
            style=Pack(
                flex=1,
                direction=COLUMN,
                padding=10
            )
        )

        # Add the content on the main window
        self.main_window.content = box

        # Show the main window
        self.main_window.show()
        self.load_demo()

def main():
    return PipeBookApp(APP_NAME, APP_ID, icon=APP_ICON)

if __name__ == '__main__':
    app = main()
    app.main_loop()



from collections import namedtuple
import toga
from toga.constants import COLUMN, ROW
from toga.sources import Source
import numpy as np
INDEX_KEY="Index"

class FrameData(Source):

    def __init__(self, raw):
        super().__init__()
        self.raw = raw
        self.columns = [c.lower() for c in raw.columns]
        self.struct = namedtuple("row", [INDEX_KEY]+self.columns)
        self.data = [self.struct(*r) for r in self.raw.itertuples()]

    def __len__(self):
        return len(self.data)

    def __getitem__(self, index):
        print('__getitem__', index)
        return self.data[index]

    def index(self, entry):
        print(INDEX_KEY, entry)
        return self.data.index(entry)



from toga.sources import TreeSource
import yaml

KEYS="level,name,typeof,size,value".split(',')

def d(l,n,t,s,v=""): return {"level": l, "name": n, "typeof": t, "size":s, "value": v}

def typeof(value): return type(value).__name__

def is_num(t): return t == "float" or t == "int" or "date" in t

def obj2tree(yml):
    s = len(yml)
    roots = [d(0,key,"dict",s) for key in yml]
    source = TreeSource(data=roots, accessors=roots[0].keys())
    for root in source._roots:
        append_dict(1, root, yml[root.name])
    return source

def append_dict(l, root, dobj):
    print(root.name)
    for k,v in dobj.items():
        t = typeof(v)
        s = len(v) if not is_num(t) else "-"
        child = root._source.append(root, **d(l, k, t, s, v))
        #print(f'dict[{l}]: {k}<{t}>')
        if ('dict' == t):
            append_dict(l+1, child, v)
        elif ('list' == t):
            append_list(l+1, child, v)

def append_list(l, root, lobj):
    print(root.name)
    for k,v in enumerate(lobj):
        t = typeof(v)
        s = len(v) if not is_num(t) else "-"
        child = root._source.append(root, **d(l, k, t, s, v))
        #print(f'list[{l}]: {k}<{t}>')
        if ('dict' == t):
            append_dict(l+1, child, v)
        elif ('list' == t):
            append_list(l+1, child, v)
